/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.example;

import java.util.Collection;
import java.util.Iterator;

import org.simantics.databoard.Accessors;
import org.simantics.databoard.Bindings;
import org.simantics.databoard.Datatypes;
import org.simantics.databoard.Datasources;
import org.simantics.databoard.Historian;
import org.simantics.databoard.accessor.Accessor;
import org.simantics.databoard.accessor.DoubleAccessor;
import org.simantics.databoard.accessor.RecordAccessor;
import org.simantics.databoard.accessor.Accessor.Listener;
import org.simantics.databoard.accessor.event.Event;
import org.simantics.databoard.accessor.interestset.InterestSet;
import org.simantics.databoard.accessor.interestset.MapInterestSet;
import org.simantics.databoard.accessor.reference.ChildReference;
import org.simantics.databoard.accessor.reference.NameReference;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.mutable.MutableVariant;
import org.simantics.databoard.datasource.ExampleDatasource;
import org.simantics.databoard.datasource.StepwiseDatasourceAccessor;
import org.simantics.databoard.history.SampleCollector;
import org.simantics.databoard.history.SamplingConfiguration;
import org.simantics.databoard.type.Datatype;

public class SubscriptionExample {

	public static void main(String[] args) throws Exception {
		
		// Create datasource. See ExampleDatasource
		ExampleDatasource datasource = new ExampleDatasource();
		
		// Open accessor to datasource
		StepwiseDatasourceAccessor datasourceAccessor = Datasources.getAccessor(datasource);
		Accessor nodes = datasourceAccessor.getComponent( new NameReference("nodes") );
		
		// Create Historian in memory
		Datatype sessionType = Historian.getRecordingSessionType();
		Binding sessionBinding = Bindings.getMutableBinding(sessionType);
		Object session = sessionBinding.createDefault();
		Accessor sessionAccessor = Accessors.getAccessor(sessionBinding, session);
		
		// Create Sample Collector
		SampleCollector collector = new SampleCollector(datasourceAccessor, sessionAccessor);
		
		// Configure collector
		SamplingConfiguration configuration = new SamplingConfiguration();
		MutableVariant keySine = MutableVariant.ofInstance("SINE");
		MutableVariant keyRamp = MutableVariant.ofInstance("RAMP");
		MutableVariant keyTime = MutableVariant.ofInstance("TIME");
		MutableVariant keyVector = MutableVariant.ofInstance("VECTOR");
		
		configuration.addSubscription( keySine, null, null );
		configuration.addSubscription( keyRamp, null, null );
		configuration.addSubscription( keyVector, null, null );
		configuration.setTimeVariable( keyTime );
		
		// Do sampling and simulate
		collector.init(configuration);
		collector.start();
		
		// Step and read value 10 times
		for (int i=0; i<10; i++) {
			System.out.println("Step "+i);
			datasource.step();
		}
		
		collector.stop();

		collector.start();
		
		// Step and read value 10 times
		for (int i=0; i<10; i++) {
			System.out.println("Step "+i);
			datasource.step();
		}
		
		collector.stop();
		
		// Un-initialize
		datasourceAccessor.dispose();
	}	
	
}
