/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.binding.reflection;

import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.UnionBinding;
import org.simantics.databoard.binding.error.BindingConstructionException;
import org.simantics.databoard.binding.error.BindingException;
import org.simantics.databoard.type.UnionType;

/**
 * Bindings an abstract class with @Union annotation to a DataBoard's UnionType.
 * 
 * Example of usage:
 *  @Union({Rectangle.class, Circle.class, Triangle.class}) class Shape {}
 *  	
 *  class Rectangle extends Shape { public int width, height; }
 *  class Circle extends Shape { public int radius; }		
 *  class Triangle extends Shape { public int sideLength; }		
 *
 * @author Toni Kalajainen
 */
class UnionClassBinding extends UnionBinding {

	Class<?>[] componentClasses;
	
	public UnionClassBinding(UnionType type) 
	throws BindingConstructionException {
		this.type = type;
	}
	
	@Override
	public Object create(int tag, Object value) {
		return value;
	}

    @Override
    public void setValue(Object union, int tag, Object value)
	throws BindingException {
    	throw new BindingException("Cannot change the class of an instance");
    }
	
	@Override
	public int getTag(Object obj) throws BindingException {
		for (int i=0; i<componentClasses.length; i++)
			if (componentClasses[i].isInstance(obj)) return i;
		throw new BindingException(obj.getClass().getSimpleName()+" is not a known component class");
	}
	@Override
	public Object getValue(Object obj) {
		return obj;
	}
	@Override
	public boolean isInstance(Object obj) {
		for (Class<?> c : componentClasses) 
			if (c.isInstance(obj)) return true;					
		return false;
	}
	
	@Override
	public boolean isImmutable() {
		return true;
	}
	
}